package com.book.dao.polardb.mapper;

import com.book.dao.polardb.entity.SunUser;
import com.book.dao.polardb.pojo.example.SunUserExample;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Mapper
@Repository
public interface Sun2Mapper {
    int selectCountUserByDay(@Param("startDate") String startDate, @Param("endDate") String endDate);

    HashMap<String, String> selectUserByDay(@Param("startDate") String startDate, @Param("endDate") String endDate);

    List<HashMap<String, String>> selectNewUserByDayChannel(@Param("startDate") String startDate, @Param("endDate") String endDate);

    List<HashMap<String, String>> selectCountByDayChannel(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 某天的新增订单，指定渠道，用户消费按用户的创建时间分组
     * static_date  new_recharge_count
     * 2021-10-01	76.00
     * 2021-10-02	1236.00
     *
     * @param startDate 开始日期
     * @param endDate   结束日期
     * @param channelId 通道标识
     * @return {@link List}<{@link HashMap}<{@link String}, {@link String}>>
     */
    List<HashMap<String, String>> selectIncomeNewUserRechargeByDayChannel(@Param("startDate") String startDate, @Param("endDate") String endDate, @Param("channelId") String channelId);


    /**
     * 某天新增用户，按照渠道分组
     * channel_id new_user_count
     * 36601	2864
     * 36603	1302
     * @param startDate 开始日期
     * @param endDate   结束日期
     * @return {@link List}<{@link HashMap}<{@link String}, {@link String}>>
     */
    List<HashMap<String, String>> selectIncomeNewUserCountByDayChannel(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 某天新增关注用户，按照渠道分组
     *
     * @param startDate 开始日期
     * @param endDate   结束日期
     * @return {@link List}<{@link HashMap}<{@link String}, {@link String}>>
     */
    List<HashMap<String, String>> selectIncomeNewUserRegisterCountByDayChannel(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 某天回收汇总
     *
     * @param dateStr str日期
     * @return {@link HashMap}<{@link String}, {@link String}>
     */
    HashMap<String, String> selectIncomeCount(@Param("dateStr") String dateStr);

    /**
     * 某天新增用户回收汇总，历史天
     *
     * @param dateStr str日期
     * @return {@link List}<{@link HashMap}<{@link String}, {@link String}>>
     */
    List<HashMap<String, String>> selectIncomeHistoryCount(@Param("dateStr") String dateStr);

    /**
     * 某天、新用户充值
     *
     * @param startDate 开始日期
     * @param endDate   结束日期
     * @return {@link String}
     */
    String selectTodayNewUserRecharge(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 指定渠道，汇总 recharge
     *
     * @param dateStr   str日期
     * @param channelId 通道标识
     * @return {@link String}
     */
    String selectIncomeRechargeByChannel(@Param("dateStr") String dateStr, @Param("channelId") String channelId);

    /**
     * 汇总 recharge
     *
     * @param dateStr   str日期
     * @return {@link String}
     */
    String selectIncomeRecharge(@Param("dateStr") String dateStr);

    /**
     * 回收历史统计
     *
     * @return {@link Map}
     */
    Map selectHistoryIncome();
}