package com.book.server.service.impl;

import java.text.DecimalFormat;
import java.util.Date;

import com.book.dao.VO.SunVO;
import com.book.dao.polardb.entity.*;
import com.book.dao.polardb.mapper.*;
import com.book.dao.polardb.pojo.example.*;
import com.book.dao.utils.TimeUtil;
import com.book.server.common.entity.PageResult;
import com.book.server.common.entity.Result;
import com.book.server.common.util.JsonUtils;
import com.book.server.config.SunTypeEnum;
import com.book.server.service.CacheService;
import com.book.server.service.SunDataService;
import com.book.server.utils.DataUtil;
import com.book.server.utils.HttpTool;
import com.book.server.utils.HttpUtils;
import com.book.server.vo.*;
import com.google.gson.FieldNamingPolicy;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * created in 2021/9/28
 * Project: book-store
 *
 * @author win7
 */
@Service
@Slf4j
public class SunDataServiceImpl implements SunDataService {
    private static final String CLIENT_ID = "10005818";
    private static final String TOKEN = "PKp2HD39ze3CdzizqH";
    private static final String VIP_ID = "36637";
    private static final String URL = "https://data.yifengaf.cn";
    public static final boolean debug = false;

    private Map<Integer, SunRequest> taskMap = new ConcurrentHashMap<>();
    @Autowired
    private SunUserCollectMapper sunUserCollectMapper;
    @Autowired
    private SunOrderMapper sunOrderMapper;

    @Autowired
    private SunOrderCollectMapper sunOrderCollectMapper;
    @Autowired
    private SunUserMapper sunUserMapper;
    @Autowired
    private SunUserActiveMapper sunUserActiveMapper;
    @Autowired
    private SunUserActive2Mapper sunUserActive2Mapper;

    @Override
    public Result requestData(SunRequest request, SunTypeEnum sunTypeEnum) {
        try {

            String url = URL + sunTypeEnum.getPath();
            request.setClient_id(CLIENT_ID);
            request.setNonce(UUID.randomUUID().toString().replace("-", ""));
            request.setTimestamp(System.currentTimeMillis() + "");
            request.setVip_id(VIP_ID);
            String signaure = DigestUtils.sha1Hex(TOKEN + request.getTimestamp() + CLIENT_ID + request.getNonce());
            request.setSignaure(signaure);

            String jsonStr = JsonUtils.toJsonStr(request);
            Map<String, Object> map = JsonUtils.getMap(jsonStr);
            String s = DataUtil.map2form(map);

            String result = HttpUtils.sendPost(url, s.getBytes(StandardCharsets.UTF_8));
            log.info("请求" + url + ":" + sunTypeEnum.getName() + ":" + JsonUtils.toJsonStr(request));
            log.info("响应" + sunTypeEnum.getName() + ":" + JsonUtils.toJsonStr(result));
            SunResult object = JsonUtils.getObject(result, SunResult.class);
            if (object != null && 0 == object.getError_code()) {
                SunResult.Data data = object.getData();
                int task_id = data.getTask_id();
                taskMap.put(task_id, request);
                return Result.success();
            } else {
                return object == null ? Result.failure() : Result.failure(object.getError_code(), object.getError_msg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("sun 获取数据异常:" + e.getMessage());
        }
        log.error(JsonUtils.toJsonStr(taskMap));
        return Result.failure();
    }

    @Override
    public void pullData(SunNotifyRequest request) {
        if (debug) {
            SunRequest object = JsonUtils.getObject("{\"client_id\":\"10005818\",\"nonce\":\"e1f1209a83e44dd1bbc6119d3d6386d7\",\"timestamp\":\"1632970800073\",\"signaure\":\"ba63fa6b6f1305ba0d29a759dad30a3dd70337a6\",\"vip_id\":\"36637\",\"start_time\":\"2021-9-30 10:00:00\",\"end_time\":\"2021-9-30 11:00:00\",\"is_finish\":\"1\"}\n", SunRequest.class);
            taskMap.put(request.getTaskId(), object);
        }

        try {


            String type = request.getType();
            String path = request.getPath();
            Integer taskId = request.getTaskId();
            if (!taskMap.containsKey(taskId)) {
                log.error("taskId不存在..." + taskId);
                return;
            }
            String url = path;
            List<String> list = HttpTool.sendGet(url);
            if (list == null || list.isEmpty()) {
                log.error("读取sun文件为空");
                return;
            }
            SunTypeEnum sunTypeEnum = SunTypeEnum.valueOf(type);

            List result = new ArrayList();

            switch (sunTypeEnum) {
                case user:
                    handleUser(list, taskId);
                    break;
                case user_active:
//                    result = caverStr2Obj(list, SunTypeUser.class);
                    handleUserActive(list, taskId);
                    break;
                case user_collect:
                case user_day_collect:
//                    result = caverStr2Obj(list, SunTypeUserDayCollect.class);
                    handleUserResult(list, taskId);
                    break;
                case channel:

                    result = caverStr2Obj(list, SunTypeChannel.class);
                    break;
                case order:
//                    result = caverStr2Obj(list, SunTypeOrder.class);
                    handleOrder(list, taskId);
                    break;
                case custom_url:
                    result = caverStr2Obj(list, SunTypeCustomUrl.class);
                    break;
                case custom_url_day_collect:
                    result = caverStr2Obj(list, SunTypeCustomUrlCollect.class);
                    break;
                case referral:
                    result = caverStr2Obj(list, SunTypeReferral.class);
                    break;
                case referral_day_collect:
                    result = caverStr2Obj(list, SunTypeReferralCollect.class);
                    break;
                case activity:
                    result = caverStr2Obj(list, SunTypeActivity.class);
                    break;
                case activity_collect:
                case activity_day_collect:

                    result = caverStr2Obj(list, SunTypeActivityCollect.class);
                    break;
                default:
                    break;
            }

        } catch (Exception e) {
            e.printStackTrace();
            log.error("sun解析数据出错:", e.getMessage());
        }
        log.info("pullData-over--" + JsonUtils.toJsonStr(taskMap));
    }

    private void handleUserActive(List<String> list, Integer taskId) {

        List<SunUserActive> result = new ArrayList<>();
        SunRequest request = taskMap.get(taskId);
        for (int i = 0; i < list.size(); i++) {
            String line = list.get(i);
            try {
                GsonBuilder gsonBuilder = new GsonBuilder();
                gsonBuilder.setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES);
                Gson gson = gsonBuilder.setDateFormat("yyyy-MM-dd HH:mm:ss").create();
                SunUserActive sunUserActive = gson.fromJson(line, SunUserActive.class);
                sunUserActive.setInsertTime(new Date());
                sunUserActive.setCollectDate(TimeUtil.dateFromStr(request.getSearch_date(), TimeUtil.YYYYMMDD1));

                result.add(sunUserActive);
            } catch (Exception e) {
                e.printStackTrace();
                log.error(e.getMessage());
            }


        }
        try {
            if (result.size() != 0) {
                SunUserActiveExample example = SunUserActiveExample.newAndCreateCriteria()
                        .andCollectDateEqualTo(TimeUtil.dateFromStr(request.getSearch_date(), TimeUtil.YYYYMMDD1)).example();
                sunUserActiveMapper.deleteByExample(example);
                sunUserActiveMapper.batchInsert(result);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
        }

        taskMap.remove(taskId);


    }

    private void handleUser(List<String> list, Integer taskId) {
        List<SunUser> result = new ArrayList<>();
        SunRequest request = taskMap.get(taskId);
        for (int i = 0; i < list.size(); i++) {
            String line = list.get(i);
            try {
                GsonBuilder gsonBuilder = new GsonBuilder();
                gsonBuilder.setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES);
                Gson gson = gsonBuilder.setDateFormat("yyyy-MM-dd HH:mm:ss").create();
                SunUser sunUser = gson.fromJson(line, SunUser.class);
                if (sunUser.getRegistertime().equals(request.getStart_time())) {
                    SunUserExample sunUserExample = SunUserExample.newAndCreateCriteria().andUserIdEqualTo(sunUser.getUserId()).example();
                    List<SunUser> sunUsers = sunUserMapper.selectByExample(sunUserExample);
                    if (sunUsers != null && sunUsers.size() > 0) {
                        continue;
                    }
                }


                sunUser.setInsertTime(new Date());
                sunUser.setStartTime(TimeUtil.dateFromStr(request.getStart_time(), TimeUtil.YYYY_MM_DD_HH_MM_SS));
                sunUser.setEndTime(TimeUtil.dateFromStr(request.getEnd_time(), TimeUtil.YYYY_MM_DD_HH_MM_SS));

                result.add(sunUser);
            } catch (Exception e) {
                e.printStackTrace();
                log.error(e.getMessage());
            }


        }

        sunUserMapper.batchInsert(result);
        taskMap.remove(taskId);


    }

    @Autowired
    private SunChannelMapper sunChannelMapper;

    @Override
    public PageResult<SunChannel> pageSunChannel(SunVO sunVO) {
        SunChannelExample example = SunChannelExample.newAndCreateCriteria().example();
        long l = sunChannelMapper.countByExample(example);
        example.page(sunVO.getPage(), sunVO.getSize());

        return new PageResult<>(l, sunChannelMapper.selectByExample(example));
    }

    @Override
    public PageResult<SunOrder> pageSunOrder(SunVO sunVO) {
        SunOrderExample example = SunOrderExample.newAndCreateCriteria().example();
        long l = sunOrderMapper.countByExample(example);
        example.page(sunVO.getPage(), sunVO.getSize());
        example.orderBy(SunOrder.Column.createTime.desc());
        return new PageResult<>(l, sunOrderMapper.selectByExample(example));
    }

    @Override
    public PageResult<SunOrderCollect> pageOrderCollect(SunVO sunVO) {
        SunOrderCollectExample example = SunOrderCollectExample.newAndCreateCriteria().example();
        long l = sunOrderCollectMapper.countByExample(example);
        example.page(sunVO.getPage(), sunVO.getSize());
        example.orderBy(SunOrderCollect.Column.startTime.desc());
        return new PageResult<>(l, sunOrderCollectMapper.selectByExample(example));
    }

    @Override
    public PageResult<SunUserCollect> pageUserCollect(SunVO sunVO) {
        SunUserCollectExample example = SunUserCollectExample.newAndCreateCriteria().example();
        long l = sunUserCollectMapper.countByExample(example);
        example.page(sunVO.getPage(), sunVO.getSize());
        example.orderBy(SunUserCollect.Column.startTime.desc());
        return new PageResult<>(l, sunUserCollectMapper.selectByExample(example));
    }

    @Autowired
    private SunStaticOrderDayMapper sunStaticOrderDayMapper;
    @Autowired
    private SunStaticOrderDay2Mapper sunStaticOrderDay2Mapper;
    @Autowired
    private SunStaticOrderDayChannelMapper sunStaticOrderDayChannelMapper;

    private int getIntFromDb(Object obj) {
        return Integer.parseInt(String.valueOf(obj));
    }

    private float getFloatFromDb(Object obj) {
        if (obj == null) {
            return 0f;
        }
        return Float.parseFloat(String.valueOf(obj));
    }

    private SunStaticOrderDay getSunOrderByDate(String startDate, String endDate) {
        int i = sunUserActive2Mapper.selectActiveCountByDate(startDate);
        if (i <= 0) {
            return null;
        }
        HashMap<String, String> map = sunUserActive2Mapper.selectRechargeByDate(startDate, endDate);
        int recharge_count = getIntFromDb(map.get("recharge_count"));
        float recharge_sum = getFloatFromDb(map.get("recharge_sum"));
        float avg = 0;
        if (recharge_count > 0) {
            avg = recharge_sum / recharge_count;
        }
        SunStaticOrderDay staticOrderDay = SunStaticOrderDay.builder()
                .staticDate(startDate)
                .activeCount(i)
                .rechargeUserCount(getIntFromDb(map.get("recharge_user_count")))
                .rechargeCount(recharge_count)
                .rechargeSum(String.valueOf(recharge_sum))
                .rechargeAvg(String.format("%.2f", avg))
                .createTime(TimeUtil.getCurrentTimestamp())
                .build();
        return staticOrderDay;
    }

    private List<SunStaticOrderDayChannel> getSunOrderByDateChannels(String startDate, String endDate) {
        List<HashMap<String, String>> activeList = sunUserActive2Mapper.selectActiveCountByDateGroupByChannel(startDate);
        List<HashMap<String, String>> recharges = sunUserActive2Mapper.selectRechargeByDateGroupByChannel(startDate, endDate);
        ArrayList<SunStaticOrderDayChannel> sunStaticOrderDayChannels = new ArrayList<>();
        for (HashMap<String, String> hashMap : activeList) {
            String channel_id = hashMap.get("channel_id");
            int active_user_count = getIntFromDb(hashMap.get("active_user_count"));
            Optional<HashMap<String, String>> optionalHashMap = recharges.stream().filter(x -> channel_id.equals(x.get("channel_id"))).findFirst();
            if (optionalHashMap.isPresent()) {
                HashMap<String, String> recharge = optionalHashMap.get();
                int recharge_count1 = getIntFromDb(recharge.get("recharge_count"));
                float recharge_sum1 = getFloatFromDb(recharge.get("recharge_sum"));
                float avg1 = 0;
                if (recharge_count1 > 0) {
                    avg1 = recharge_sum1 / recharge_count1;
                }
                SunStaticOrderDayChannel dayChannel = SunStaticOrderDayChannel.builder()
                        .staticDate(startDate)
                        .channelId(recharge.get("channel_id"))
                        .channelName(recharge.get("channel_name"))
                        .activeCount(active_user_count)
                        .rechargeUserCount(getIntFromDb(recharge.get("recharge_user_count")))
                        .rechargeSum(String.valueOf(recharge_sum1))
                        .rechargeCount(recharge_count1)
                        .rechargeAvg(String.format("%.2f", avg1))
                        .createTime(TimeUtil.getCurrentTimestamp())
                        .build();
                sunStaticOrderDayChannels.add(dayChannel);
            } else {
                SunStaticOrderDayChannel dayChannel = SunStaticOrderDayChannel.builder()
                        .staticDate(startDate)
                        .channelId(channel_id)
                        .channelName(cacheService.getChannelName(channel_id))
                        .activeCount(active_user_count)
                        .rechargeUserCount(0)
                        .rechargeSum("0")
                        .rechargeCount(0)
                        .rechargeAvg("0")
                        .createTime(TimeUtil.getCurrentTimestamp())
                        .build();
                sunStaticOrderDayChannels.add(dayChannel);
            }
        }

        return sunStaticOrderDayChannels;
    }

    @Override
    public void orderStaticDate(String startDate, String endDate) {
        /*
         * 清除指定日期的统计，因为可能重新计算
         */
        sunStaticOrderDayMapper.deleteByExample(SunStaticOrderDayExample.newAndCreateCriteria().andStaticDateEqualTo(startDate).example());
        sunStaticOrderDayChannelMapper.deleteByExample(SunStaticOrderDayChannelExample.newAndCreateCriteria().andStaticDateEqualTo(startDate).example());

        /*
        开始统计
         */
        // 先不分渠道
        SunStaticOrderDay staticOrderDay = getSunOrderByDate(startDate, endDate);
        if (staticOrderDay == null) {
            return;
        } else {
            sunStaticOrderDayMapper.insert(staticOrderDay);
        }
        /*
        分渠道统计
         */
        sunStaticOrderDayChannelMapper.batchInsert(getSunOrderByDateChannels(startDate, endDate));
    }

    @Override
    public SunStaticOrderDay getTodayOrderStatic() {
        return getSunOrderByDate(TimeUtil.getTodayStr(), TimeUtil.getTomorrowStr());
    }

    @Override
    public SunStaticOrderDay getYesterdayOrderStatic() {
        return sunStaticOrderDayMapper.selectOneByExample(SunStaticOrderDayExample.newAndCreateCriteria()
                .andStaticDateEqualTo(TimeUtil.getYesterdayStr()).example());
    }

    @Override
    public Map getHistoryOrderStatic() {
        return sunStaticOrderDay2Mapper.selectHistory();
    }

    @Override
    public List<SunStaticOrderDayChannel> getTodayOrderStaticByChannel() {
        return getSunOrderByDateChannels(TimeUtil.getTodayStr(), TimeUtil.getTomorrowStr());
    }

    @Override
    public PageResult<SunStaticOrderDay> getOrderStatics(int page, int size) {
        SunStaticOrderDayExample example = SunStaticOrderDayExample.newAndCreateCriteria().example();
        long l = sunStaticOrderDayMapper.countByExample(example);
        example.page(page, size);
        example.orderBy(SunStaticOrderDay.Column.staticDate.desc());
        List<SunStaticOrderDay> sunStaticOrderDays = sunStaticOrderDayMapper.selectByExample(example);
        return new PageResult<>(l, sunStaticOrderDays);
    }

    @Override
    public List<SunStaticOrderDay> getOrderStaticsAll() {
        SunStaticOrderDayExample example = SunStaticOrderDayExample.newAndCreateCriteria().example();
        List<SunStaticOrderDay> sunStaticOrderDays = sunStaticOrderDayMapper.selectByExample(example);
        return sunStaticOrderDays;
    }

    @Override
    public List<SunStaticOrderDayChannel> getOrderStaticByDateAndChannel(String date) {
        return sunStaticOrderDayChannelMapper.selectByExample(
                SunStaticOrderDayChannelExample.newAndCreateCriteria()
                        .andStaticDateEqualTo(date)
                        .example()
        );
    }

    @Autowired
    private SunStaticUserDayMapper sunStaticUserDayMapper;
    @Autowired
    private SunStaticUserDayChannelMapper sunStaticUserDayChannelMapper;
    @Autowired
    private Sun2Mapper sun2Mapper;

    @Override
    public void userStaticYesterday(String yesterdayStr, String todayStr) {
        /*
         * 清除指定日期的统计，因为可能重新计算
         */
        sunStaticUserDayMapper.deleteByExample(SunStaticUserDayExample.newAndCreateCriteria().andStaticDateEqualTo(yesterdayStr).example());
        sunStaticUserDayChannelMapper.deleteByExample(SunStaticUserDayChannelExample.newAndCreateCriteria().andStaticDateEqualTo(yesterdayStr).example());
        /*
         * 不分渠道统计
         */
        SunStaticUserDay sunUserByDate = getSunUserByDate(yesterdayStr, todayStr);
        if (sunUserByDate == null) {
            return;
        } else {
            sunStaticUserDayMapper.insert(sunUserByDate);
        }
        /*
         * 分渠道统计
         */
        sunStaticUserDayChannelMapper.batchInsert(getSunUserByDateChannels(yesterdayStr, todayStr));

    }

    @Override
    public SunStaticUserDay getTodayUserStatic() {
        return getSunUserByDate(TimeUtil.getTodayStr(), TimeUtil.getTomorrowStr());
    }

    @Override
    public List<SunStaticUserDayChannel> getTodayUserStaticByChannel() {
        return getSunUserByDateChannels(TimeUtil.getTodayStr(), TimeUtil.getTomorrowStr());
    }

    @Override
    public SunStaticUserDay getYesterdayUserStatic() {
        return sunStaticUserDayMapper.selectOneByExample(SunStaticUserDayExample.newAndCreateCriteria().andStaticDateEqualTo(TimeUtil.getYesterdayStr()).example());
    }

    @Override
    public Map getHistoryUserStatic() {
        return sunStaticOrderDay2Mapper.selectUserHistory();
    }

    @Override
    public PageResult<SunStaticUserDay> getUserStatics(int page, int size) {
        SunStaticUserDayExample example = SunStaticUserDayExample.newAndCreateCriteria().example();
        long l = sunStaticUserDayMapper.countByExample(example);
        example.page(page, size);
        example.orderBy(SunStaticOrderDay.Column.staticDate.desc());
        List<SunStaticUserDay> sunStaticUserDays = sunStaticUserDayMapper.selectByExample(example);
        return new PageResult<>(l, sunStaticUserDays);
    }

    @Override
    public List<SunStaticUserDayChannel> getUserStaticByDateAndChannel(String date) {
        return sunStaticUserDayChannelMapper.selectByExample(
                SunStaticUserDayChannelExample.newAndCreateCriteria()
                        .andStaticDateEqualTo(date)
                        .example()
        );
    }

    @Override
    public List<SunStaticUserDay> getUserStaticsAll() {
        SunStaticUserDayExample example = SunStaticUserDayExample.newAndCreateCriteria().example();
        return sunStaticUserDayMapper.selectByExample(example);
    }

    @Autowired
    private SunStaticIncomeMapper sunStaticIncomeMapper;
    @Autowired
    private SunStaticIncomeChannelMapper sunStaticIncomeChannelMapper;
    @Autowired
    private SunStaticIncomeHundredMapper sunStaticIncomeHundredMapper;
    @Autowired
    private SunStaticIncomeHundredChannelMapper sunStaticIncomeHundredChannelMapper;

    @Override
    public void incomeStatic(String dateStr) throws ParseException {
        String nextDay = TimeUtil.getNextDay(dateStr);
        // 新增用户数,按渠道汇总
        List<HashMap<String, String>> countMap = sun2Mapper.selectIncomeNewUserCountByDayChannel(dateStr, nextDay);
        // 新增充值金额（元）,按渠道汇总
        List<HashMap<String, String>> registerMap = sun2Mapper.selectIncomeNewUserRegisterCountByDayChannel(dateStr, nextDay);
        //所有渠道信息
        List<SunChannel> sunChannels = sunChannelMapper.selectByExample(SunChannelExample.newAndCreateCriteria().example());
        HashSet<String> resetChargeDates = new HashSet<>();
        for (SunChannel sunChannel : sunChannels) {
            //当天所有充值，按照用户创建日期 汇总
            List<HashMap<String, String>> hashMaps = sun2Mapper.selectIncomeNewUserRechargeByDayChannel(dateStr, nextDay, sunChannel.getChannelId());
            String todayRechargeSum = "0";

            for (HashMap<String, String> hashMap : hashMaps) {
                String static_date = hashMap.get("static_date");
                String new_recharge_sum = String.format("%.2f", getFloatFromDb(hashMap.get("new_recharge_sum")));
                if (dateStr.equals(static_date)) {
                    todayRechargeSum = new_recharge_sum;
                }
                // 第几天
                int day = TimeUtil.diffDays(dateStr, static_date);
                if (day > 365) {
                    continue;
                }
                resetChargeDates.add(static_date);
                //插入历史用户充值统计
                sunStaticIncomeHundredChannelMapper.insert(SunStaticIncomeHundredChannel.builder()
                        .staticDate(static_date)
                        .channelId(sunChannel.getChannelId())
                        .rechargeSum(new_recharge_sum)
                        .dayNum(day)
                        .insertDate(dateStr)
                        .build());
                //重新 渠道-日期 历史天的累计
                if (static_date.equals(dateStr)) {
                    computerChanelDateRecharge(sunChannel, static_date);
                }
            }

            SunStaticIncomeChannel sunStaticIncomeChannel = new SunStaticIncomeChannel();
            sunStaticIncomeChannel.setStaticDate(dateStr);
            sunStaticIncomeChannel.setChannelId(sunChannel.getChannelId());
            sunStaticIncomeChannel.setChannelName(sunChannel.getChannelName());
            sunStaticIncomeChannel.setBookName(getChannelBookName(sunChannel.getChannelId()));
            sunStaticIncomeChannel.setNewUserCount(Integer.valueOf(getNumValueFromListMap(countMap, "channel_id", sunChannel.getChannelId(), "new_user_count")));
            sunStaticIncomeChannel.setNewFollowCount(Integer.valueOf(getNumValueFromListMap(registerMap, "channel_id", sunChannel.getChannelId(), "new_follow_count")));
            sunStaticIncomeChannel.setNewRechargeSum(todayRechargeSum);
            sunStaticIncomeChannel.setFirstRoi("");
            //当天的消耗
            sunStaticIncomeChannel.setAllSpend("");
            // 整体 ROI，会变动
            sunStaticIncomeChannel.setAllRoi("");
            //累计充值总额，会变动
            sunStaticIncomeChannel.setAllRecharge(todayRechargeSum);
            sunStaticIncomeChannel.setInsertTime(new Date());
            sunStaticIncomeChannel.setUpdateTime(new Date());

            sunStaticIncomeChannelMapper.insert(sunStaticIncomeChannel);

            // 再统计当天的
            computerChanelDateRecharge(sunChannel, dateStr);
        }

        //再统计当天，group 渠道即可
        HashMap<String, String> incomeMap = sun2Mapper.selectIncomeCount(dateStr);
        if (incomeMap != null) {
            SunStaticIncome sunStaticIncome = new SunStaticIncome();
            sunStaticIncome.setStaticDate(dateStr);
            sunStaticIncome.setNewUserCount(getIntFromDb(incomeMap.get("new_user_count")));
            sunStaticIncome.setNewFollowCount(getIntFromDb(incomeMap.get("new_follow_count")));
            sunStaticIncome.setNewRechargeSum(String.format("%.2f", getFloatFromDb(incomeMap.get("new_recharge_sum"))));
            sunStaticIncome.setAllSpend(String.format("%.2f", getFloatFromDb(incomeMap.get("all_spend"))));
            sunStaticIncome.setFirstRoi(""); // setNewRechargeSum / setAllSpend
            sunStaticIncome.setAllRecharge(String.format("%.2f", getFloatFromDb(incomeMap.get("all_recharge"))));
            sunStaticIncome.setAllRoi("");  // setAllRecharge / setAllSpend
            sunStaticIncome.setInsertTime(new Date());
            sunStaticIncome.setUpdateTime(new Date());
            sunStaticIncomeMapper.insert(sunStaticIncome);
        }

        //对于今天新增的数据，插入指定日期和天数
        List<HashMap<String, String>> historyMaps = sun2Mapper.selectIncomeHistoryCount(dateStr);
        ArrayList<SunStaticIncomeHundred> sunStaticIncomeHundreds = new ArrayList<>();
        for (HashMap<String, String> map : historyMaps) {
            SunStaticIncomeHundred incomeHundred = new SunStaticIncomeHundred();
            incomeHundred.setStaticDate(map.get("static_date"));
            incomeHundred.setRechargeSum(String.format("%.2f", getFloatFromDb(map.get("recharge_sum"))));
            incomeHundred.setDayNum(getIntFromDb(map.get("day_num")));
            incomeHundred.setInsertDate(dateStr);
            sunStaticIncomeHundreds.add(incomeHundred);
        }
        sunStaticIncomeHundredMapper.batchInsert(sunStaticIncomeHundreds);


        //重新统计指定天的累计数据
        for (String resetChargeDate : resetChargeDates) {
            computerDateRecharge(resetChargeDate);
        }


    }

    /**
     * 重新统计 指定渠道-指定天的 充值积累情况
     *
     * @param sunChannel  太阳频道
     * @param static_date 静态的日期
     */
    private void computerChanelDateRecharge(SunChannel sunChannel, String static_date) {
        SunStaticIncomeChannel sunStaticIncomeChannel = sunStaticIncomeChannelMapper.selectOneByExample(SunStaticIncomeChannelExample.newAndCreateCriteria().andStaticDateEqualTo(static_date).andChannelIdEqualTo(sunChannel.getChannelId()).example());
        if (sunStaticIncomeChannel == null) {
            log.error("没有找到历史统计数据：" + sunChannel.getChannelId() + "--" + static_date);
            return;
        }
        String s = sun2Mapper.selectIncomeRechargeByChannel(static_date, sunChannel.getChannelId());
        if (s == null) {
            s = "0";
        }
        sunStaticIncomeChannel.setAllRecharge(s);
        // todo 需要更新ROI
        sunStaticIncomeChannelMapper.updateByPrimaryKeySelective(sunStaticIncomeChannel, SunStaticIncomeChannel.Column.allRecharge);
    }

    /**
     * 重新统计 指定指定天的 充值积累情况
     *
     * @param static_date 静态的日期
     */
    private void computerDateRecharge(String static_date) {
        SunStaticIncome sunStaticIncome = sunStaticIncomeMapper.selectOneByExample(SunStaticIncomeExample.newAndCreateCriteria().andStaticDateEqualTo(static_date).example());
        if (sunStaticIncome == null) {
            log.error("没有找到历史统计数据：" + static_date);
            return;
        }
        String s = sun2Mapper.selectIncomeRecharge(static_date);
        if (s == null) {
            s = "0";
        }
        sunStaticIncome.setAllRecharge(s);
        // todo 需要更新ROI
        sunStaticIncomeMapper.updateByPrimaryKeySelective(sunStaticIncome, SunStaticIncome.Column.allRecharge);
    }

    /**
     * 获取渠道推广书的名字
     *
     * @param channelId 通道标识
     * @return {@link String}
     * @throws ParseException 解析异常
     */
    private String getChannelBookName(String channelId) throws ParseException {
        SunStaticIncomeChannelExample example = SunStaticIncomeChannelExample.newAndCreateCriteria()
                .andChannelIdEqualTo(channelId)
                .example();
        example.orderBy(SunStaticIncomeChannel.Column.staticDate.desc());
        SunStaticIncomeChannel preDayChannel = sunStaticIncomeChannelMapper.selectOneByExample(example);
        String bookName = "";
        if (preDayChannel != null) {
            bookName = preDayChannel.getBookName();
        }
        return bookName;
    }

    private String getNumValueFromListMap(List<HashMap<String, String>> list, String k1, String v1, String k2) {
        for (HashMap<String, String> map : list) {
            if (v1.equals(map.get(k1))) {
                return String.valueOf(map.get(k2));
            }
        }
        return "0";
    }

    @Override
    public void incomeStaticDate(String dateStr) throws ParseException {
        sunStaticIncomeHundredChannelMapper.deleteByExample(SunStaticIncomeHundredChannelExample.newAndCreateCriteria().andInsertDateEqualTo(dateStr).example());
        sunStaticIncomeChannelMapper.deleteByExample(SunStaticIncomeChannelExample.newAndCreateCriteria().andStaticDateEqualTo(dateStr).example());
        sunStaticIncomeMapper.deleteByExample(SunStaticIncomeExample.newAndCreateCriteria().andStaticDateEqualTo(dateStr).example());
        sunStaticIncomeHundredMapper.deleteByExample(SunStaticIncomeHundredExample.newAndCreateCriteria().andInsertDateEqualTo(dateStr).example());
        incomeStatic(dateStr);

    }

    @Override
    public SunStaticIncome getTodayIncomeStatic() {
        String aLong = String.valueOf(sun2Mapper.selectTodayNewUserRecharge(TimeUtil.getTodayStr(), TimeUtil.getTomorrowStr()));
        SunStaticIncome sunStaticIncome = new SunStaticIncome();
        sunStaticIncome.setNewRechargeSum(String.valueOf(aLong));
        sunStaticIncome.setAllSpend("");
        sunStaticIncome.setFirstRoi("");
        return sunStaticIncome;
    }

    @Override
    public SunStaticIncome getYesterdayIncomeStatic() {
        return sunStaticIncomeMapper.selectOneByExample(SunStaticIncomeExample.newAndCreateCriteria().andStaticDateEqualTo(TimeUtil.getYesterdayStr()).example());
    }

    @Override
    public SunStaticIncome getHistoryIncomeStatic() {
        Map map = sun2Mapper.selectHistoryIncome();
        if (map == null) {
            return null;
        }
        float all_recharge = getFloatFromDb(map.get("all_recharge"));
        float all_spend = getFloatFromDb(map.get("all_spend"));
        float roi = 0f;
        if (all_spend != 0) {
            roi = all_recharge / all_spend;
        }
        return SunStaticIncome.builder()
                .allRecharge(String.format("%.2f", all_recharge))
                .allSpend(String.format("%.2f", all_spend))
                .allRoi(String.format("%.2f", roi))
                .build();
    }

    @Override
    public Map getIncomeByDay(int page, int size) {
        HashMap<String, Object> resMap = new HashMap<>();
        SunStaticIncomeExample example = SunStaticIncomeExample.newAndCreateCriteria().example();
        // 总共多少条
        long l = sunStaticIncomeMapper.countByExample(example);
        resMap.put("total", l);
        // 排序查询指定页
        example.orderBy(SunStaticIncome.Column.staticDate.desc());
        example.page(page, size);
        List<SunStaticIncome> sunStaticIncomes = sunStaticIncomeMapper.selectByExample(example);
        //保留小数点
        //for (SunStaticIncome sunStaticIncome : sunStaticIncomes) {
        //    DecimalFormat df = new DecimalFormat("#.00");//此为保留2位小数
        //    sunStaticIncome.setAllRecharge(df.format(sunStaticIncome.getAllRecharge()));
        //}

        //查询另一张表对应的前 60 天的数据
        List<String> collect = sunStaticIncomes.stream().map(SunStaticIncome::getStaticDate).collect(Collectors.toList());
        List<SunStaticIncomeHundred> sunStaticIncomeHundreds = sunStaticIncomeHundredMapper.selectByExample(
                SunStaticIncomeHundredExample.newAndCreateCriteria().andStaticDateIn(collect).andDayNumLessThan(61).example());


        ArrayList<Object> objects = new ArrayList<>();
        for (SunStaticIncome sunStaticIncome : sunStaticIncomes) {
            HashMap<String, Object> map = new HashMap<>();
            map.put("income", sunStaticIncome);
            map.put("incomeDay", sunStaticIncomeHundreds.stream().filter(x -> sunStaticIncome.getStaticDate().equals(x.getStaticDate())).collect(Collectors.toList()));
            objects.add(map);
        }
        resMap.put("list", objects);
        return resMap;
    }

    @Override
    public Map getIncomeByDayChannel(int page, int size, String channelId) {
        HashMap<String, Object> resMap = new HashMap<>();
        SunStaticIncomeChannelExample example = SunStaticIncomeChannelExample.newAndCreateCriteria().andChannelIdEqualTo(channelId).example();
        //总共多少条
        long l = sunStaticIncomeChannelMapper.countByExample(example);
        resMap.put("total", l);
        // 排序查询指定页
        example.orderBy(SunStaticIncomeChannel.Column.staticDate.desc());
        example.page(page, size);
        List<SunStaticIncomeChannel> sunStaticIncomes = sunStaticIncomeChannelMapper.selectByExample(example);

        //查询另一张表对应的前 60 天的数据
        List<String> collect = sunStaticIncomes.stream().map(SunStaticIncomeChannel::getStaticDate).collect(Collectors.toList());
        List<SunStaticIncomeHundredChannel> sunStaticIncomeHundreds = sunStaticIncomeHundredChannelMapper.selectByExample(
                SunStaticIncomeHundredChannelExample.newAndCreateCriteria().andStaticDateIn(collect).andChannelIdEqualTo(channelId).andDayNumLessThan(61).example());

        ArrayList<Object> objects = new ArrayList<>();
        for (SunStaticIncomeChannel sunStaticIncome : sunStaticIncomes) {
            HashMap<String, Object> map = new HashMap<>();
            map.put("income", sunStaticIncome);
            map.put("incomeDay", sunStaticIncomeHundreds.stream().filter(x -> sunStaticIncome.getStaticDate().equals(x.getStaticDate())).collect(Collectors.toList()));
            objects.add(map);
        }
        resMap.put("list", objects);
        return resMap;
    }

    @Override
    public Boolean updateChannelName(String channelId, String bookName, String staticDate) {
        int i = sunStaticIncomeChannelMapper.updateByExampleSelective(
                SunStaticIncomeChannel.builder()
                        .bookName(bookName)
                        .build(),
                SunStaticIncomeChannelExample.newAndCreateCriteria()
                        .andChannelIdEqualTo(channelId)
                        .andStaticDateEqualTo(staticDate)
                        .example(),
                SunStaticIncomeChannel.Column.bookName
        );
        return i > 0;
    }

    @Override
    public List<SunChannel> getAllChannel() {
        return sunChannelMapper.selectByExample(SunChannelExample.newAndCreateCriteria().example());
    }

    private SunStaticUserDay getSunUserByDate(String startDate, String endDate) {
        int i = sun2Mapper.selectCountUserByDay(startDate, endDate);
        if (i <= 0) {
            return null;
        }
        HashMap<String, String> recharge = sun2Mapper.selectUserByDay(startDate, endDate);
        int new_recharge_user = getIntFromDb(recharge.get("new_recharge_user"));
        float new_recharge_sum = getFloatFromDb(recharge.get("new_recharge_sum"));
        float avg = 0;
        //付费率
        float rate = 0;
        if (new_recharge_sum > 0) {
            avg = new_recharge_sum / new_recharge_user;
            rate = (float) new_recharge_user / i;
        }
        SunStaticUserDay day = SunStaticUserDay.builder()
                .staticDate(startDate)
                .newAllUser(i)
                .newRechargeUser(new_recharge_user)
                .newRechargeSum(String.format("%.2f", new_recharge_sum))
                .newRechargeCount(getIntFromDb(recharge.get("new_recharge_count")))
                .rechargeAvg(String.format("%.2f", avg))
                .rechargeRate(String.format("%.2f", rate))
                .insertTime(TimeUtil.getCurrentTimestamp())
                .price("0") // TODO
                .build();
        return day;
    }

    private List<SunStaticUserDayChannel> getSunUserByDateChannels(String startDate, String endDate) {
        List<HashMap<String, String>> list = sun2Mapper.selectCountByDayChannel(startDate, endDate);
        List<HashMap<String, String>> recharges = sun2Mapper.selectNewUserByDayChannel(startDate, endDate);
        ArrayList<SunStaticUserDayChannel> sunStaticDayChannels = new ArrayList<>();

        for (HashMap<String, String> hashMap : list) {
            String channel_id = hashMap.get("channel_id");
            int user_count = getIntFromDb(hashMap.get("user_count"));
            Optional<HashMap<String, String>> optionalHashMap = recharges.stream().filter(x -> x.get("channel_id").equals(channel_id)).findFirst();

            if (optionalHashMap.isPresent()) {
                HashMap<String, String> recharge = optionalHashMap.get();

                int new_recharge_user = getIntFromDb(recharge.get("new_recharge_user"));
                float new_recharge_sum = getFloatFromDb(recharge.get("new_recharge_sum"));
                float avg = 0;
                //付费率
                float rate = 0;
                if (new_recharge_sum > 0) {
                    avg = new_recharge_sum / new_recharge_user;
                    rate = (float) new_recharge_user / user_count;
                }
                SunStaticUserDayChannel day = SunStaticUserDayChannel.builder()
                        .staticDate(startDate)
                        .channelId(recharge.get("channel_id"))
                        .channelName(recharge.get("channel_name"))
                        .newAllUser(user_count)
                        .newRechargeUser(new_recharge_user)
                        .newRechargeSum(String.format("%.2f", new_recharge_sum))
                        .newRechargeCount(getIntFromDb(recharge.get("new_recharge_count")))
                        .rechargeAvg(String.format("%.2f", avg))
                        .rechargeRate(String.format("%.2f", rate))
                        .insertTime(TimeUtil.getCurrentTimestamp())
                        .price("0") // TODO
                        .build();
                sunStaticDayChannels.add(day);
            } else {
                SunStaticUserDayChannel day = SunStaticUserDayChannel.builder()
                        .staticDate(startDate)
                        .channelId(channel_id)
                        .channelName(cacheService.getChannelName(channel_id))
                        .newAllUser(user_count)
                        .newRechargeUser(0)
                        .newRechargeSum("0")
                        .newRechargeCount(0)
                        .rechargeAvg("0")
                        .rechargeRate("0")
                        .insertTime(TimeUtil.getCurrentTimestamp())
                        .price("0") // TODO
                        .build();
                sunStaticDayChannels.add(day);
            }
        }
        return sunStaticDayChannels;
    }

    @Autowired
    private CacheService cacheService;

    private void handleOrder(List<String> list, Integer taskId) {
        SunRequest request = taskMap.get(taskId);
        List<SunOrder> result = new ArrayList<>();
        Map<String, SunOrderCollect> collectMap = new HashMap();

        for (int i = 0; i < list.size(); i++) {
            try {


                String line = list.get(i);
                if (StringUtils.isEmpty(line)) {
                    continue;
                }
                GsonBuilder gsonBuilder = new GsonBuilder();
                gsonBuilder.setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES);
                Gson gson = gsonBuilder.setDateFormat("yyyy-MM-dd HH:mm:ss").create();
                SunOrder sunOrder = gson.fromJson(line, SunOrder.class);
                if (TimeUtil.dateToStr(sunOrder.getCreateTime(), TimeUtil.YYYY_MM_DD_HH_MM_SS).equals(request.getStart_time())) {
                    SunOrderExample sunOrderExample = SunOrderExample.newAndCreateCriteria().andMerchantIdEqualTo(sunOrder.getMerchantId()).example();
                    List<SunOrder> sunOrders = sunOrderMapper.selectByExample(sunOrderExample);
                    if (sunOrder != null && sunOrders.size() > 0) {
                        continue;
                    }
                }


                sunOrder.setInsertTime(new Date());
                SunOrderCollect sunOrderCollect = collectMap.get(sunOrder.getChannelId());
                if (sunOrderCollect == null) {
                    sunOrderCollect = new SunOrderCollect();
                    sunOrderCollect.setOrderNum(0);
                    sunOrderCollect.setMoney(new BigDecimal(0.00));
                    sunOrderCollect.setVipMoney(new BigDecimal(0.00));
                    sunOrderCollect.setOrderNum(0);
                    sunOrderCollect.setSuccessOrderNum(new Integer(0));
                    sunOrderCollect.setCreateTime(new Date());
                    sunOrderCollect.setChannelId(sunOrder.getChannelId());
                    sunOrderCollect.setType(0);
                    sunOrderCollect.setStartTime(TimeUtil.dateFromStr(request.getStart_time(), TimeUtil.YYYY_MM_DD_HH_MM_SS));
                    sunOrderCollect.setEndTime(TimeUtil.dateFromStr(request.getEnd_time(), TimeUtil.YYYY_MM_DD_HH_MM_SS));

                }
                sunOrderCollect.setOrderNum(sunOrderCollect.getOrderNum() + 1);
                if ("完成".equals(sunOrder.getState())) {
                    sunOrderCollect.setSuccessOrderNum(sunOrderCollect.getSuccessOrderNum() + 1);

                    //统计累加
                    if ("书币充值".equals(sunOrder.getType())) {

                        sunOrderCollect.setMoney(sunOrderCollect.getMoney().add(sunOrder.getMoney()));
                    } else if ("VIP充值".equals(sunOrder.getType())) {
                        sunOrderCollect.setVipMoney(sunOrderCollect.getVipMoney().add(sunOrder.getMoney()));
                    }
                }
                result.add(sunOrder);
                collectMap.put(sunOrder.getChannelId(), sunOrderCollect);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        try {
            List<SunOrderCollect> sunOrderCollects = new ArrayList<SunOrderCollect>(collectMap.values());
            sunOrderCollectMapper.batchInsert(sunOrderCollects);
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
        }


        sunOrderMapper.batchInsert(result);

        taskMap.remove(taskId);


    }

    /**
     * 用户数据处理
     *
     * @param list
     * @param taskId
     */
    private void handleUserResult(List<String> list, Integer taskId) {
        List<SunUserCollect> result = new ArrayList<>();
        SunRequest request = taskMap.get(taskId);
        for (int i = 0; i < list.size(); i++) {
            String line = list.get(i);
            try {
                GsonBuilder gsonBuilder = new GsonBuilder();
                gsonBuilder.setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES);
                Gson gson = gsonBuilder.setDateFormat("yyyy-MM-dd HH:mm:ss").create();
                SunUserCollect sunUserCollect = gson.fromJson(line, SunUserCollect.class);
                sunUserCollect.setCreateTime(new Date());
                sunUserCollect.setStartTime(TimeUtil.dateFromStr(request.getStart_time(), TimeUtil.YYYY_MM_DD_HH_MM_SS));
                sunUserCollect.setEndTime(TimeUtil.dateFromStr(request.getEnd_time(), TimeUtil.YYYY_MM_DD_HH_MM_SS));
                sunUserCollect.setType(0);

                result.add(sunUserCollect);
            } catch (Exception e) {
                e.printStackTrace();
                log.error(e.getMessage());
            }


        }

        sunUserCollectMapper.batchInsert(result);
        taskMap.remove(taskId);


    }


    private List caverStr2Obj(List<String> list, Class classes) {

        List result = new ArrayList<>();

        for (int i = 0; i < list.size(); i++) {
            String line = list.get(i);
            Object object = JsonUtils.getObject(line, classes);
            result.add(object);
        }
        return result;
    }

    public static void main(String[] args) {
       /* String line = "{\"merchant_id\":\"20210930105902_643505930_Sbjp\",\"transaction_id\":null,\"type\":\"书币充值\",\"money\":\"66.0\",\"state\":\"未完成\",\"create_time\":\"2021-09-30 10:59:03\",\"finish_time\":null,\"book_id\":null,\"book_name\":null,\"book_tags\":null,\"referral_id\":\"11539446\",\"referral_url\":\"https://wx6ba3406d7a8bddbd.yifengab.cn/t/11539446\",\"user_id\":\"643505930\",\"channel_id\":\"36648\",\"subscribe_time\":\"2021-09-30 09:16:08\",\"user_createtime\":\"2021-09-30 09:16:08\",\"openid\":\"ogjqN5g2cQlvpJPwtupxIq7NPXf8\",\"ip\":\"171.104.15.31\",\"user_agent\":\"Mozilla/5.0 (iPhone; CPU iPhone OS 14_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 MicroMessenger/8.0.7(0x18000733) NetType/4G Language/zh_CN\",\"activity_id\":\"7574690\",\"register_ip\":\"171.104.15.31\",\"recharge_money\":null,\"recharge_count\":null}\n";
        GsonBuilder gsonBuilder = new GsonBuilder();
        gsonBuilder.setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES);
        Gson gson = gsonBuilder.setDateFormat("yyyy-MM-dd HH:mm:ss").create();
        SunOrder sunOrder = gson.fromJson(line, SunOrder.class);
        System.out.println(sunOrder);*/
        Map map = new HashMap();
        map.put(new Integer(1), "sss");
        System.out.println(map.containsKey(new Integer(1)));
    }
}